      module output
      integer :: logmode = 0
      end module

      module constants
      double precision :: rc = -8.0d-10 ! Cutoff radius [m]
      double precision :: dt = 1.0d-15 ! Time step [s]
      double precision :: a  = 4.04959d-10 ! Lattice constant [m]
      double precision :: scl_lat = 1.0d0 ! Scaling factor (lattice const.)
      character (2), allocatable :: asp(:) ! Atom species (name)
!      integer, allocatable :: atp(:) ! Atom species (numbering in code)
      integer :: nsp ! number of species
      integer :: nesp(50) ! number of atoms for each species
      character(2) :: aesp(50) ! Atom species name for each species
      integer :: multisystem = 0 
      character (15) :: potential_func = 'EAM_MIS' ! Potential type
      character (7) :: potential_arg
      double precision :: am = 1.6726485d-27 ! Mass of proton [kg]
      double precision :: bm = 1.6749543d-27 ! Mass of neutron [kg]
      double precision, allocatable :: wm(:) ! Mass of atom [kg]
      double precision :: wmave ! Averaged mass of atom
      double precision :: pi = 3.141592654d0 ! Pi
      double precision :: bl = 1.380662d-23 ! Boltzmann constant
      double precision :: ev = 1.6021892d-19 ! 1eV=1.6021892d-19 J
      double precision :: elc= 1.6021892d-19  ! elemetary charge e (C)
      double precision :: epsil0=8.854187816d-12 ! Permittivity in vacuum (F/m)
      double precision :: bohr=5.2917706d-11
      double precision :: ang= 1.0d-10 ! 1ang=10^-10 m
      double precision :: gpa= 1.0d9 ! 1GPa=10^9 Pa
      double precision :: mpa= 1.0d6 ! 1MPa=10^6 Pa
      double precision :: relax_factor = 1.0d0
!      double precision :: c11,c12,c13,c23,c22,c33 ! elastic constants
      double precision :: elasc(6,6) ! elastic constants
      double precision :: param(100) ! supplementary parameters for potential

     end module

      module bookkeep_param
      double precision :: frc = 9.0d-10
      integer :: nbk=10
      integer, allocatable :: idpt(:,:),knum(:),irep(:)
      integer :: nbook = 1000000
      integer :: nrpx = 1, nrpy = 1, nrpz = 1 ! # of replica cells to be serched
      integer :: bookkeep_algo = 1 ! Bookkeeping algorithm (1=normal, 2=double)
      end module

      module rp_param
      logical :: rpart = .false.
      integer, allocatable :: ncl(:,:,:),nir(:,:),idpart(:)
      integer :: npcx=8, npcy=3, npcz=3
      integer :: nnor, nnir      
      double precision :: spcx, spcy, spcz
      double precision, allocatable :: coor(:,:)
      end module

      module debug_variables
      logical :: debug = .false.
      double precision, allocatable :: den1(:),den2(:)
      integer, allocatable :: nn(:),nnnum(:,:)
      double precision, allocatable :: distance(:,:),fcom(:,:,:)
      double precision :: debug_val(10)
      end module

      module variables
      integer :: iwarn = 0      ! Count of warning(s)
      integer :: step = 0       ! Iteration number
      integer :: step_ite =50   ! Iteration to go
      integer :: step_start     ! Start step
      integer :: step_end       ! Last step
      integer :: n              ! Number of atoms
      integer :: vscale = 0     ! Velocity scaling
                                ! 0: no, integer:interval steps to apply
      integer :: damper = 0     ! Atom damping
                                ! 0: no, integer:interval steps to apply
      integer :: dump_frequency = 0 ! frequency of writing .cfg files
      integer :: meam_stress_frequency = 0 ! frequency of calculating stress in MEAM
      integer :: naccew = 10    ! Accuracy parameter for Ewald method
      logical :: cellconstx = .false. ! Cell constraint
      logical :: cellconsty = .false. 
      logical :: cellconstz = .false. 

      integer :: algorithm = 8  ! Numerical int. algo.
        ! 0=Velocity Verlet, NVE (Allen & Tildesley)
        ! 1=Gear's P-C, NVE (Allen & Tildesley)
        ! 2=Gear, NVT (Nose thermostat)
        ! 3=Gear, NPH (Andersen pressure control)
        ! 4=Gear, NPT (Nose and Andersen)
        ! 5=Gear, NPH-PR (Parrinello-Rahman)
        ! 6=Gear, NPT-PR (Nose and Parrinello-Rahman)
        ! 7=Stress relaxation by annealing, no atom motion
        ! 8=Stress relaxation by annealing, atom motion by verlet
        ! 50=Gear, Stress relaxation by NPH-PR & breaking
        ! 100=Configuration relaxation by NR
        ! 101=Atom relaxation by CG (Conjugate Gradient) method
        ! 102=Atom relaxation by FIRE (Bitzek et al. PRL97-170201)
        ! 103=Atom and cell relaxation by FIRE
        ! 104=Atom and cell relaxation by FIRE (test phase)
!      double precision, allocatable :: ityp(:) ! Atom type
      double precision, allocatable :: rx(:),ry(:),rz(:) ! Atom positions
      double precision, allocatable :: vx(:),vy(:),vz(:) ! Velocities
      double precision, allocatable :: fx(:),fy(:),fz(:) ! Forces
      double precision, allocatable :: qx(:),qy(:),qz(:) ! Normalized pos
      double precision, allocatable :: ax(:),ay(:),az(:) ! 2nd derivatives
      double precision, allocatable :: bx(:),by(:),bz(:) ! 3rd derivatives
      double precision, allocatable :: cx(:),cy(:),cz(:) ! 4th derivatives
      character (1), allocatable :: mfx(:),mfy(:),mfz(:) ! Atom motion control
      double precision, allocatable :: qpx(:),qpy(:),qpz(:) ! Atom positions for memory
      double precision, allocatable :: tmprx(:,:),tmpry(:,:),tmprz(:,:) ! positions for SM
      double precision, allocatable :: tmpfx(:,:),tmpfy(:,:),tmpfz(:,:) ! forces for SM
      double precision, allocatable :: enpoteach(:)      ! Potential per atom
      double precision, allocatable :: enkineach(:)      ! Kinetic energy per atom
      double precision, allocatable :: satom(:,:,:) ! Atomic stress
      double precision :: s,sv,sa,sb,sc ! S and its derivatives
      double precision :: sf ! "correct" sa (to calculate corrector)
      double precision :: v,vv,va,vb,vc ! Volume and its deriatives
      double precision :: hmat(3,3),hvmat(3,3),hamat(3,3),hbmat(3,3),hcmat(3,3)
                                ! Lattice matrix and its derivatives
      double precision :: hmat0(3,3) ! Original lattice matrix (for constraint)
      double precision :: hinmat(3,3) ! Inverse Lattice matrix
      double precision :: dmat(3,3) ! for stress calculation
      double precision :: enpot,enkin ! potenrial & kinetic energies
      double precision, allocatable :: fconst(:,:,:,:,:) ! force constant matrix (Nx3xNx3xL)
      double complex, allocatable :: dynmat(:,:) ! dynamical matrix (3Nx3N)
      logical :: pbcx = .true. ,pbcy = .true. ,pbcz = .true.
                                ! Periodic boundary condition
      logical :: continue_md = .false.
      logical :: ini_velocity = .false.
      integer :: ran_seed = -999

      integer :: multicellx = 1, multicelly = 1, multicellz = 1 ! to multiply cell size

      logical :: neb = .false.
      integer :: neb_num = 10, neb_ite = 10000
      double precision :: neb_tol = 1.0e-11, neb_dt = 1.0d-14
      
      
!!! Potential parameters
      double precision :: potparam1 = 0.0d0, potparam2 = 0.0d0
      double precision :: potparam3 = 0.0d0, potparam4 = 0.0d0

!!! CG (T.Shimada 21.02.07)
      double precision, allocatable :: rstx(:),rsty(:),rstz(:)
      double precision, allocatable :: sgx(:,:),sgy(:,:),sgz(:,:)
      double precision, allocatable :: shx(:,:),shy(:,:),shz(:,:)
      double precision :: lam_init = 1.0d-3
      double precision :: normsh,lamcrr,lamtr
      double precision :: cgpot0,cggrad0,cgpot1,cggrad1
!!! FIRE (T.Shimada 21.02.07)
      integer :: ifire = 0
      double precision :: fire_alph = 0.10d0
      double precision :: ffdtmax

! Temperature
      double precision :: temp_set = 200.0d0 ! target temperature
      double precision :: temp_weight = 1.0d0 ! Weight of thermostat
      double precision :: wqr = 1.5d1
      double precision :: wq, temp
      integer :: appl_temp_step = -1, appl_temp_length = -1 ! step# and step length to apply temperature
      double precision :: appl_temp_temp1 = 0.0d0, appl_temp_temp2 = 0.0d0 ! applied temp. and release temp.

! Stress
      double precision :: virx, viry, virz
      double precision :: sgm_set = 0.0d0 ! target stress
      double precision :: sxx=0.0d6, syy=0.0d6, szz=0.0d6 ! target stress
      double precision :: syz=0.0d6, szx=0.0d6, sxy=0.0d6 ! target stress
      logical :: dstress = .false.
      integer :: dstress_start = 0
      double precision :: dsxx=0.0d6, dsyy=0.0d6, dszz=0.0d6 ! increment of stress
      double precision :: dsyz=0.0d6, dszx=0.0d6, dsxy=0.0d6 ! increment of stress
      double precision :: sgmmat_set(3,3) = 0.0d0 ! target stress
      double precision :: stress_weight = 1.0d0 ! Weight of piston
      double precision :: wwr = 0.005d1
      double precision :: ww, pex, pexmat(3,3)
                                ! Weight of piston, target pressure (value, tensor)
      double precision :: pres, sgm ! Calculated pressure, stress (value)
      double precision :: sgmmat(3,3) = 0.0d0 ! Calculated stress tensor
      double precision :: sgmmat_ave(3,3) = 0.0d0
      double precision, allocatable :: sgmmat_prv(:,:,:) ! to calculate time-averaged stress
      integer, parameter :: average_span_max=1000, average_span_min=10
      integer :: average_span = 100
      double precision :: prdamper = 0.0d0 ! damper parameter for algo = 50
      logical :: prdamper2 = .false. ! another damper for algo = 50
      double precision :: prlimit  = -100.0d0 ! limiter parameter for algo = 50
      logical :: fixrotation = .false. ! to restrict cell rotation or not
      logical :: fixdrift = .true. ! to restrict drift of atoms
      double precision :: cntx, cnty, cntz  ! center of atom coordinates
      double precision :: cntpx,cntpy,cntpz ! center of atom coordinates

! Elastic constants evaluation
      double precision :: econstd

! Input
      double precision :: strmat_set(3,3) ! Strain matrix

! Tolerance (MD stop policy)
! Negative value for ignorance
      double precision :: tolene = -1.0d0 ! Energy tolerance (Averaged eV per atom)
      double precision :: tolfor = -1.0d0 ! Force tolerance (Maximum F, eV/A)
      double precision :: tolst = -1.0d0 ! Stress tolerance (Mpa)
      double precision :: ene0, ene1, force0, force1 ! Values used for tolerance check
      double precision :: st0, st1 ! Values used for tolerance check
      integer :: loopflg

! Instability flag
      logical :: inst = .false.
      integer :: inst_drawvector_maxmode = 5
      integer :: inst_curve_maxmode = 5
      logical :: inst_curve = .false.
      double precision :: inst_curve_range = 2.0d0
      integer :: inst_curve_mesh = 100
      logical :: inst_decom = .false.
      integer :: inst_decom_nfile = 1
      logical :: inst_decom_curve = .false.
      integer :: inst_decom_maxmode = 50
      logical :: inst_surf = .false.
      integer :: inst_surf_x = 1, inst_surf_y = 2
      double precision :: inst_surf_xmn = 0.5d0, inst_surf_xmx = 0.5d0
      double precision :: inst_surf_ymn = 0.5d0, inst_surf_ymx = 0.5d0
      double precision :: inst_surf_gridx = 0.04d0, inst_surf_gridy = 0.04d0
! Voronoi polyhedron analysis, radius
      double precision :: rc_vor

      integer :: idbg
      double precision :: dbg

! Diffusion analysis
      logical :: diffusion = .false.
      integer :: difout_int = 1000
      double precision, allocatable :: rxd(:),ryd(:),rzd(:)
!      double precision, allocatable :: rxp(:),ryp(:),rzp(:)
      double precision :: msdx, msdy, msdz, msdxc, msdyc, msdzc

! Time count
      real :: atime,btime(2)

! For debugging
!      double precision :: strmat_store(3,3),hmat_store(3,3)
!      double precision,allocatable :: rx_store(:),ry_store(:),rz_store(:)

      end module variables

